@@ -1,5 +1,5 @@
 /*
-    $Id: strscan.c 23721 2009-06-17 04:57:11Z matz $
+    $Id: strscan.c 34053 2011-12-15 05:48:39Z nobu $
 
     Copyright (c) 1999-2006 Minero Aoki
 
@@ -11,6 +11,7 @@
 #include "ruby/ruby.h"
 #include "ruby/re.h"
 #include "ruby/encoding.h"
+#include "regint.h"
 
 #define STRSCAN_VERSION "0.7.0"
 
@@ -29,7 +30,7 @@ struct strscanner
 
     /* the string to scan */
     VALUE str;
-    
+
     /* scan pointers */
     long prev;   /* legal only when MATCHED_P(s) */
     long curr;   /* always legal */
@@ -51,8 +52,8 @@ struct strscanner
 #define EOS_P(s) ((s)->curr >= RSTRING_LEN(p->str))
 
 #define GET_SCANNER(obj,var) do {\
-    Data_Get_Struct(obj, struct strscanner, var);\
-    if (NIL_P(var->str)) rb_raise(rb_eArgError, "uninitialized StringScanner object");\
+    (var) = check_strscan(obj);\
+    if (NIL_P((var)->str)) rb_raise(rb_eArgError, "uninitialized StringScanner object");\
 } while (0)
 
 /* =======================================================================
@@ -63,9 +64,10 @@ static VALUE infect _((VALUE str, struct strscanner *p));
 static VALUE extract_range _((struct strscanner *p, long beg_i, long end_i));
 static VALUE extract_beg_len _((struct strscanner *p, long beg_i, long len));
 
-void check_strscan _((VALUE obj));
-static void strscan_mark _((struct strscanner *p));
-static void strscan_free _((struct strscanner *p));
+static struct strscanner *check_strscan _((VALUE obj));
+static void strscan_mark _((void *p));
+static void strscan_free _((void *p));
+static size_t strscan_memsize _((const void *p));
 static VALUE strscan_s_allocate _((VALUE klass));
 static VALUE strscan_initialize _((int argc, VALUE *argv, VALUE self));
 static VALUE strscan_init_copy _((VALUE vself, VALUE vorig));
@@ -157,29 +159,47 @@ extract_beg_len(struct strscanner *p, long beg_i, long len)
    ======================================================================= */
 
 static void
-strscan_mark(struct strscanner *p)
+strscan_mark(void *ptr)
 {
+    struct strscanner *p = ptr;
     rb_gc_mark(p->str);
 }
 
 static void
-strscan_free(struct strscanner *p)
+strscan_free(void *ptr)
 {
+    struct strscanner *p = ptr;
     onig_region_free(&(p->regs), 0);
     ruby_xfree(p);
 }
 
+static size_t
+strscan_memsize(const void *ptr)
+{
+    const struct strscanner *p = ptr;
+    size_t size = 0;
+    if (p) {
+	size = sizeof(*p) - sizeof(p->regs) + onig_region_memsize(&p->regs);
+    }
+    return size;
+}
+
+static const rb_data_type_t strscanner_type = {
+    "StringScanner",
+    {strscan_mark, strscan_free, strscan_memsize}
+};
+
 static VALUE
 strscan_s_allocate(VALUE klass)
 {
     struct strscanner *p;
-    
+
     p = ALLOC(struct strscanner);
     MEMZERO(p, struct strscanner, 1);
     CLEAR_MATCH_STATUS(p);
     onig_region_init(&(p->regs));
     p->str = Qnil;
-    return Data_Wrap_Struct(klass, strscan_mark, strscan_free, p);
+    return TypedData_Wrap_Struct(klass, &strscanner_type, p);
 }
 
 /*
@@ -194,7 +214,7 @@ strscan_initialize(int argc, VALUE *argv, VALUE self)
     struct strscanner *p;
     VALUE str, need_dup;
 
-    Data_Get_Struct(self, struct strscanner, p);
+    p = check_strscan(self);
     rb_scan_args(argc, argv, "11", &str, &need_dup);
     StringValue(str);
     p->str = str;
@@ -202,14 +222,10 @@ strscan_initialize(int argc, VALUE *argv, VALUE self)
     return self;
 }
 
-void
+static struct strscanner *
 check_strscan(VALUE obj)
 {
-    if (TYPE(obj) != T_DATA || RDATA(obj)->dmark != (RUBY_DATA_FUNC)strscan_mark) {
-        rb_raise(rb_eTypeError,
-                 "wrong argument type %s (expected StringScanner)",
-                 rb_obj_classname(obj));
-    }
+    return rb_check_typeddata(obj, &strscanner_type);
 }
 
 /*
@@ -224,9 +240,8 @@ strscan_init_copy(VALUE vself, VALUE vorig)
 {
     struct strscanner *self, *orig;
 
-    Data_Get_Struct(vself, struct strscanner, self);
-    check_strscan(vorig);
-    Data_Get_Struct(vorig, struct strscanner, orig);
+    self = check_strscan(vself);
+    orig = check_strscan(vorig);
     if (self != orig) {
 	self->flags = orig->flags;
 	self->str = orig->str;
@@ -317,12 +332,10 @@ strscan_get_string(VALUE self)
 static VALUE
 strscan_set_string(VALUE self, VALUE str)
 {
-    struct strscanner *p;
+    struct strscanner *p = check_strscan(self);
 
-    Data_Get_Struct(self, struct strscanner, p);
     StringValue(str);
-    p->str = rb_str_dup(str);
-    rb_obj_freeze(p->str);
+    p->str = str;
     p->curr = 0;
     CLEAR_MATCH_STATUS(p);
     return str;
@@ -354,9 +367,9 @@ strscan_concat(VALUE self, VALUE str)
 }
 
 /*
- * Returns the position of the scan pointer.  In the 'reset' position, this
+ * Returns the byte position of the scan pointer.  In the 'reset' position, this
  * value is zero.  In the 'terminated' position (i.e. the string is exhausted),
- * this value is the length of the string.
+ * this value is the bytesize of the string.
  *
  * In short, it's a 0-based index into the string.
  *
@@ -379,7 +392,7 @@ strscan_get_pos(VALUE self)
 /*
  * call-seq: pos=(n)
  *
- * Modify the scan pointer.
+ * Set the byte position of the scan pointer.
  *
  *   s = StringScanner.new('test string')
  *   s.pos = 7            # -> 7
@@ -406,7 +419,7 @@ strscan_do_scan(VALUE self, VALUE regex, int succptr, int getstr, int headonly)
     regex_t *rb_reg_prepare_re(VALUE re, VALUE str);
     struct strscanner *p;
     regex_t *re;
-    int ret;
+    long ret;
     int tmpreg;
 
     Check_Type(regex, T_REGEXP);
@@ -610,7 +623,7 @@ strscan_exist_p(VALUE self, VALUE re)
  *
  *   s = StringScanner.new("Fri Dec 12 1975 14:39")
  *   s.skip_until /12/           # -> 10
- *   s                           # 
+ *   s                           #
  */
 static VALUE
 strscan_skip_until(VALUE self, VALUE re)
@@ -656,7 +669,7 @@ static void
 adjust_registers_to_matched(struct strscanner *p)
 {
     onig_region_clear(&(p->regs));
-    onig_region_set(&(p->regs), 0, 0, p->curr - p->prev);
+    onig_region_set(&(p->regs), 0, 0, (int)(p->curr - p->prev));
 }
 
 /*
@@ -894,7 +907,7 @@ strscan_matched_p(VALUE self)
 
 /*
  * Returns the last matched string.
- * 
+ *
  *   s = StringScanner.new('test string')
  *   s.match?(/\w+/)     # -> 4
  *   s.matched           # -> "test"
@@ -952,7 +965,7 @@ strscan_aref(VALUE self, VALUE idx)
 
     GET_SCANNER(self, p);
     if (! MATCHED_P(p))        return Qnil;
-    
+
     i = NUM2LONG(idx);
     if (i < 0)
         i += p->regs.num_regs;
@@ -1068,7 +1081,7 @@ strscan_inspect(VALUE self)
     long len;
     VALUE a, b;
 
-    Data_Get_Struct(self, struct strscanner, p);
+    p = check_strscan(self);
     if (NIL_P(p->str)) {
         len = snprintf(buf, BUFSIZE, "#<%s (uninitialized)>",
                        rb_class2name(CLASS_OF(self)));
@@ -1142,20 +1155,20 @@ inspect2(struct strscanner *p)
 
 /*
  * Document-class: StringScanner
- * 
+ *
  * StringScanner provides for lexical scanning operations on a String.  Here is
  * an example of its usage:
  *
  *   s = StringScanner.new('This is an example string')
  *   s.eos?               # -> false
- *   
+ *
  *   p s.scan(/\w+/)      # -> "This"
  *   p s.scan(/\w+/)      # -> nil
  *   p s.scan(/\s+/)      # -> " "
  *   p s.scan(/\s+/)      # -> nil
  *   p s.scan(/\w+/)      # -> "is"
  *   s.eos?               # -> false
- *   
+ *
  *   p s.scan(/\s+/)      # -> " "
  *   p s.scan(/\w+/)      # -> "an"
  *   p s.scan(/\s+/)      # -> " "
@@ -1163,7 +1176,7 @@ inspect2(struct strscanner *p)
  *   p s.scan(/\s+/)      # -> " "
  *   p s.scan(/\w+/)      # -> "string"
  *   s.eos?               # -> true
- *   
+ *
  *   p s.scan(/\s+/)      # -> nil
  *   p s.scan(/\w+/)      # -> nil
  *
@@ -1192,7 +1205,7 @@ inspect2(struct strscanner *p)
  * the string without actually scanning.  You can access the most recent match.
  * You can modify the string being scanned, reset or terminate the scanner,
  * find out or change the position of the scan pointer, skip ahead, and so on.
- * 
+ *
  * === Advancing the Scan Pointer
  *
  * - #getch
@@ -1223,7 +1236,7 @@ inspect2(struct strscanner *p)
  * - #reset
  * - #terminate
  * - #pos=
- * 
+ *
  * === Match Data
  *
  * - #matched
@@ -1257,10 +1270,10 @@ Init_strscan()
     tmp = rb_str_new2(STRSCAN_VERSION);
     rb_obj_freeze(tmp);
     rb_const_set(StringScanner, rb_intern("Version"), tmp);
-    tmp = rb_str_new2("$Id: strscan.c 23721 2009-06-17 04:57:11Z matz $");
+    tmp = rb_str_new2("$Id: strscan.c 34053 2011-12-15 05:48:39Z nobu $");
     rb_obj_freeze(tmp);
     rb_const_set(StringScanner, rb_intern("Id"), tmp);
-    
+
     rb_define_alloc_func(StringScanner, strscan_s_allocate);
     rb_define_private_method(StringScanner, "initialize", strscan_initialize, -1);
     rb_define_private_method(StringScanner, "initialize_copy", strscan_init_copy, 1);
