@@ -1,21 +1,20 @@
+# encoding: utf-8
 ######################################################################
 # This file is imported from the minitest project.
 # DO NOT make modifications in this repo. They _will_ be reverted!
 # File a patch instead and assign it to Ryan Davis.
 ######################################################################
 
-require 'stringio'
 require 'pathname'
-require 'minitest/unit'
-
-MiniTest::Unit.autorun
+require 'minitest/metametameta'
 
 module MyModule; end
 class AnError < StandardError; include MyModule; end
+class ImmutableString < String; def inspect; super.freeze; end; end
 
-class TestMiniTestUnit < MiniTest::Unit::TestCase
-  pwd = Pathname.new(File.expand_path(Dir.pwd))
-  basedir = Pathname.new(File.expand_path(MiniTest::MINI_DIR)) + 'mini'
+class TestMiniTestUnit < MetaMetaMetaTestCase
+  pwd = Pathname.new File.expand_path Dir.pwd
+  basedir = Pathname.new(File.expand_path "lib/minitest") + 'mini'
   basedir = basedir.relative_path_from(pwd).to_s
   MINITEST_BASE_DIR = basedir[/\A\./] ? basedir : "./#{basedir}"
   BT_MIDDLE = ["#{MINITEST_BASE_DIR}/test.rb:161:in `each'",
@@ -23,38 +22,6 @@ class TestMiniTestUnit < MiniTest::Unit::TestCase
                "#{MINITEST_BASE_DIR}/test.rb:139:in `run'",
                "#{MINITEST_BASE_DIR}/test.rb:106:in `run'"]
 
-  def assert_report expected = nil
-    expected ||= "Run options: --seed 42
-
-# Running tests:
-
-.
-
-Finished tests in 0.00
-
-1 tests, 1 assertions, 0 failures, 0 errors, 0 skips
-"
-    output = @output.string.sub(/Finished tests in .*/, "Finished tests in 0.00")
-    output.sub!(/Loaded suite .*/, 'Loaded suite blah')
-    output.sub!(/^(\s+)(?:#{Regexp.union(__FILE__, File.expand_path(__FILE__))}):\d+:/o, '\1FILE:LINE:')
-    output.sub!(/\[(?:#{Regexp.union(__FILE__, File.expand_path(__FILE__))}):\d+\]/o, '[FILE:LINE]')
-    assert_equal(expected, output)
-  end
-
-  def setup
-    srand 42
-    MiniTest::Unit::TestCase.reset
-    @tu = MiniTest::Unit.new
-    @output = StringIO.new("")
-    MiniTest::Unit.output = @output
-  end
-
-  def teardown
-    MiniTest::Unit.output = $stdout
-    MiniTest::Unit.runner = nil
-    Object.send :remove_const, :ATestCase if defined? ATestCase
-  end
-
   def test_class_puke_with_assertion_failed
     exception = MiniTest::Assertion.new "Oh no!"
     exception.set_backtrace ["unhappy"]
@@ -185,12 +152,38 @@ Finished tests in 0.00
     bt = util_expand_bt bt
 
     ex = ["-e:1"]
-    fu = MiniTest::filter_backtrace(bt)
+    fu = MiniTest::filter_backtrace bt
     assert_equal ex, fu
   end
 
+  def test_run_test
+    Class.new MiniTest::Unit::TestCase do
+      attr_reader :foo
+
+      def run_test name
+        @foo = "hi mom!"
+        super
+        @foo = "okay"
+      end
+
+      def test_something
+        assert_equal "hi mom!", foo
+      end
+    end
+
+    expected = clean <<-EOM
+      .
+
+      Finished tests in 0.00
+
+      1 tests, 1 assertions, 0 failures, 0 errors, 0 skips
+    EOM
+
+    assert_report expected
+  end
+
   def test_run_error
-    tc = Class.new(MiniTest::Unit::TestCase) do
+    Class.new MiniTest::Unit::TestCase do
       def test_something
         assert true
       end
@@ -200,30 +193,24 @@ Finished tests in 0.00
       end
     end
 
-    Object.const_set(:ATestCase, tc)
+    expected = clean <<-EOM
+      E.
 
-    @tu.run %w[--seed 42]
+      Finished tests in 0.00
 
-    expected = "Run options: --seed 42
+        1) Error:
+      test_error(#<Class:0xXXX>):
+      RuntimeError: unhandled exception
+          FILE:LINE:in `test_error'
 
-# Running tests:
+      2 tests, 1 assertions, 0 failures, 1 errors, 0 skips
+    EOM
 
-E.
-
-Finished tests in 0.00
-
-  1) Error:
-test_error(ATestCase):
-RuntimeError: unhandled exception
-    FILE:LINE:in `test_error'
-
-2 tests, 1 assertions, 0 failures, 1 errors, 0 skips
-"
     assert_report expected
   end
 
   def test_run_error_teardown
-    tc = Class.new(MiniTest::Unit::TestCase) do
+    Class.new MiniTest::Unit::TestCase do
       def test_something
         assert true
       end
@@ -233,30 +220,24 @@ RuntimeError: unhandled exception
       end
     end
 
-    Object.const_set(:ATestCase, tc)
+    expected = clean <<-EOM
+      E
 
-    @tu.run %w[--seed 42]
+      Finished tests in 0.00
 
-    expected = "Run options: --seed 42
+        1) Error:
+      test_something(#<Class:0xXXX>):
+      RuntimeError: unhandled exception
+          FILE:LINE:in `teardown'
 
-# Running tests:
+      1 tests, 1 assertions, 0 failures, 1 errors, 0 skips
+    EOM
 
-E
-
-Finished tests in 0.00
-
-  1) Error:
-test_something(ATestCase):
-RuntimeError: unhandled exception
-    FILE:LINE:in `teardown'
-
-1 tests, 1 assertions, 0 failures, 1 errors, 0 skips
-"
     assert_report expected
   end
 
   def test_run_failing
-    tc = Class.new(MiniTest::Unit::TestCase) do
+    Class.new MiniTest::Unit::TestCase do
       def test_something
         assert true
       end
@@ -266,29 +247,23 @@ RuntimeError: unhandled exception
       end
     end
 
-    Object.const_set(:ATestCase, tc)
+    expected = clean <<-EOM
+      F.
 
-    @tu.run %w[--seed 42]
+      Finished tests in 0.00
 
-    expected = "Run options: --seed 42
+        1) Failure:
+      test_failure(#<Class:0xXXX>) [FILE:LINE]:
+      Failed assertion, no message given.
 
-# Running tests:
+      2 tests, 2 assertions, 1 failures, 0 errors, 0 skips
+    EOM
 
-F.
-
-Finished tests in 0.00
-
-  1) Failure:
-test_failure(ATestCase) [FILE:LINE]:
-Failed assertion, no message given.
-
-2 tests, 2 assertions, 1 failures, 0 errors, 0 skips
-"
     assert_report expected
   end
 
   def test_run_failing_filtered
-    tc = Class.new(MiniTest::Unit::TestCase) do
+    Class.new MiniTest::Unit::TestCase do
       def test_something
         assert true
       end
@@ -298,39 +273,37 @@ Failed assertion, no message given.
       end
     end
 
-    Object.const_set(:ATestCase, tc)
+    expected = clean <<-EOM
+      .
 
-    @tu.run %w[--name /some|thing/ --seed 42]
+      Finished tests in 0.00
 
-    expected = "Run options: --name \"/some|thing/\" --seed 42
+      1 tests, 1 assertions, 0 failures, 0 errors, 0 skips
+    EOM
 
-# Running tests:
-
-.
-
-Finished tests in 0.00
-
-1 tests, 1 assertions, 0 failures, 0 errors, 0 skips
-"
-    assert_report expected
+    assert_report expected, %w[--name /some|thing/ --seed 42]
   end
 
   def test_run_passing
-    tc = Class.new(MiniTest::Unit::TestCase) do
+    Class.new MiniTest::Unit::TestCase do
       def test_something
         assert true
       end
     end
 
-    Object.const_set(:ATestCase, tc)
+    expected = clean <<-EOM
+      .
 
-    @tu.run %w[--seed 42]
+      Finished tests in 0.00
 
-    assert_report
+      1 tests, 1 assertions, 0 failures, 0 errors, 0 skips
+    EOM
+
+    assert_report expected
   end
 
   def test_run_skip
-    tc = Class.new(MiniTest::Unit::TestCase) do
+    Class.new MiniTest::Unit::TestCase do
       def test_something
         assert true
       end
@@ -340,25 +313,19 @@ Finished tests in 0.00
       end
     end
 
-    Object.const_set(:ATestCase, tc)
+    expected = clean <<-EOM
+      S.
 
-    @tu.run %w[--seed 42]
+      Finished tests in 0.00
 
-    expected = "Run options: --seed 42
+      2 tests, 1 assertions, 0 failures, 0 errors, 1 skips
+    EOM
 
-# Running tests:
-
-S.
-
-Finished tests in 0.00
-
-2 tests, 1 assertions, 0 failures, 0 errors, 1 skips
-"
     assert_report expected
   end
 
   def test_run_skip_verbose
-    tc = Class.new(MiniTest::Unit::TestCase) do
+    Class.new MiniTest::Unit::TestCase do
       def test_something
         assert true
       end
@@ -368,27 +335,21 @@ Finished tests in 0.00
       end
     end
 
-    Object.const_set(:ATestCase, tc)
-
-    @tu.run %w[--seed 42 --verbose]
-
-    expected = "Run options: --seed 42 --verbose
-
-# Running tests:
-
-ATestCase#test_skip = 0.00 s = S
-ATestCase#test_something = 0.00 s = .
+    expected = clean <<-EOM
+      #<Class:0xXXX>#test_skip = 0.00 s = S
+      #<Class:0xXXX>#test_something = 0.00 s = .
 
 
-Finished tests in 0.00
+      Finished tests in 0.00
 
-  1) Skipped:
-test_skip(ATestCase) [FILE:LINE]:
-not yet
+        1) Skipped:
+      test_skip(#<Class:0xXXX>) [FILE:LINE]:
+      not yet
 
-2 tests, 1 assertions, 0 failures, 0 errors, 1 skips
-"
-    assert_report expected
+      2 tests, 1 assertions, 0 failures, 0 errors, 1 skips
+    EOM
+
+    assert_report expected, %w[--seed 42 --verbose]
   end
 
   def test_default_runner_is_minitest_unit
@@ -396,18 +357,15 @@ not yet
   end
 
   def test_run_with_other_runner
-
-    runner = Class.new(MiniTest::Unit) do
-      # Run once before each suite
-      def _run_suite(suite, type)
-        begin
-          suite.before_suite
-          super(suite, type)
-        end
+    MiniTest::Unit.runner = Class.new MiniTest::Unit do
+      def _run_suite suite, type
+        suite.before_suite # Run once before each suite
+        super suite, type
       end
-    end
+    end.new
 
-    tc = Class.new(MiniTest::Unit::TestCase) do
+    Class.new MiniTest::Unit::TestCase do
+      def self.name; "wacky!" end
 
       def self.before_suite
         MiniTest::Unit.output.puts "Running #{self.name} tests"
@@ -423,27 +381,173 @@ not yet
       end
     end
 
-    Object.const_set(:ATestCase, tc)
-    MiniTest::Unit.runner = runner.new
-    @tu.run %w[--seed 42]
+    expected = clean <<-EOM
+      Running wacky! tests
+      ..
 
-    # We should only see 'running ATestCase tests' once
-    expected = "Run options: --seed 42
+      Finished tests in 0.00
 
-# Running tests:
+      2 tests, 2 assertions, 0 failures, 0 errors, 0 skips
+    EOM
 
-Running ATestCase tests
-..
-
-Finished tests in 0.00
-
-2 tests, 2 assertions, 0 failures, 0 errors, 0 skips
-"
     assert_report expected
   end
 
+  def with_overridden_include
+    Class.class_eval do
+      def inherited_with_hacks klass
+        throw :inherited_hook
+      end
+
+      alias inherited_without_hacks inherited
+      alias inherited               inherited_with_hacks
+      alias IGNORE_ME!              inherited # 1.8 bug. god I love venture bros
+    end
+
+    yield
+  ensure
+    Class.class_eval do
+      alias inherited inherited_without_hacks
+
+      undef_method :inherited_with_hacks
+      undef_method :inherited_without_hacks
+    end
+
+    refute_respond_to Class, :inherited_with_hacks
+    refute_respond_to Class, :inherited_without_hacks
+  end
+
+  def test_inherited_hook_plays_nice_with_others
+    with_overridden_include do
+      assert_throws :inherited_hook do
+        Class.new MiniTest::Unit::TestCase
+      end
+    end
+  end
+
+  def test_before_setup
+    call_order = []
+    Class.new MiniTest::Unit::TestCase do
+      define_method :setup do
+        super()
+        call_order << :setup
+      end
+
+      define_method :before_setup do
+        call_order << :before_setup
+      end
+
+      def test_omg; assert true; end
+    end
+
+    @tu.run %w[--seed 42]
+
+    expected = [:before_setup, :setup]
+    assert_equal expected, call_order
+  end
+
+  def test_passed_eh_teardown_good
+    test_class = Class.new MiniTest::Unit::TestCase do
+      def teardown; assert true; end
+      def test_omg; assert true; end
+    end
+
+    test = test_class.new :test_omg
+    test.run @tu
+    assert test.passed?
+  end
+
+  def test_passed_eh_teardown_flunked
+    test_class = Class.new MiniTest::Unit::TestCase do
+      def teardown; flunk;       end
+      def test_omg; assert true; end
+    end
+
+    test = test_class.new :test_omg
+    test.run @tu
+    refute test.passed?
+  end
+
+  def test_after_teardown
+    call_order = []
+    Class.new MiniTest::Unit::TestCase do
+      define_method :teardown do
+        super()
+        call_order << :teardown
+      end
+
+      define_method :after_teardown do
+        call_order << :after_teardown
+      end
+
+      def test_omg; assert true; end
+    end
+
+    @tu.run %w[--seed 42]
+
+    expected = [:teardown, :after_teardown]
+    assert_equal expected, call_order
+  end
+
+  def test_all_teardowns_are_guaranteed_to_run
+    call_order = []
+    Class.new MiniTest::Unit::TestCase do
+      define_method :after_teardown do
+        super()
+        call_order << :after_teardown
+        raise
+      end
+
+      define_method :teardown do
+        super()
+        call_order << :teardown
+        raise
+      end
+
+      define_method :before_teardown do
+        super()
+        call_order << :before_teardown
+        raise
+      end
+
+      def test_omg; assert true; end
+    end
+
+    @tu.run %w[--seed 42]
+
+    expected = [:before_teardown, :teardown, :after_teardown]
+    assert_equal expected, call_order
+  end
+
+  def test_setup_and_teardown_survive_inheritance
+    call_order = []
+
+    parent = Class.new MiniTest::Spec do
+      before do
+        call_order << :setup_method
+      end
+
+      after do
+        call_order << :teardown_method
+      end
+
+      define_method :test_something do
+        call_order << :test
+      end
+    end
+
+    _ = Class.new parent
+
+    @tu.run %w[--seed 42]
+
+    # Once for the parent class, once for the child
+    expected = [:setup_method, :test, :teardown_method] * 2
+
+    assert_equal expected, call_order
+  end
+
   def util_expand_bt bt
-    if RUBY_VERSION =~ /^1\.9/ then
+    if RUBY_VERSION >= '1.9.0' then
       bt.map { |f| (f =~ /^\./) ? File.expand_path(f) : f }
     else
       bt
@@ -452,7 +556,11 @@ Finished tests in 0.00
 end
 
 class TestMiniTestUnitTestCase < MiniTest::Unit::TestCase
+  RUBY18 = ! defined? Encoding
+
   def setup
+    super
+
     MiniTest::Unit::TestCase.reset
 
     @tc = MiniTest::Unit::TestCase.new 'fake tc'
@@ -463,7 +571,6 @@ class TestMiniTestUnitTestCase < MiniTest::Unit::TestCase
   def teardown
     assert_equal(@assertion_count, @tc._assertions,
                  "expected #{@assertion_count} assertions to be fired during the test, not #{@tc._assertions}") if @tc._assertions
-    Object.send :remove_const, :ATestCase if defined? ATestCase
   end
 
   def test_assert
@@ -485,15 +592,25 @@ class TestMiniTestUnitTestCase < MiniTest::Unit::TestCase
   end
 
   def test_assert_block
-    @tc.assert_block do
-      true
+    exp = ["NOTE: MiniTest::Unit::TestCase#assert_block is deprecated,",
+           "use assert. It will be removed on 2013-01-01."].join " "
+
+    out, err = capture_io do
+      @tc.assert_block do
+        true
+      end
     end
+
+    assert_equal "", out
+    assert_match exp, err
   end
 
   def test_assert_block_triggered
-    util_assert_triggered "blah.\nExpected block to return true value." do
-      @tc.assert_block "blah" do
-        false
+    assert_output do
+      util_assert_triggered "blah.\nExpected block to return true value." do
+        @tc.assert_block "blah" do
+          false
+        end
       end
     end
   end
@@ -630,13 +747,13 @@ class TestMiniTestUnitTestCase < MiniTest::Unit::TestCase
   end
 
   def test_assert_in_delta_triggered
-    util_assert_triggered 'Expected 0.0 - 0.001 (0.001) to be < 1.0e-06.' do
+    util_assert_triggered 'Expected |0.0 - 0.001| (0.001) to be < 1.0e-06.' do
       @tc.assert_in_delta 0.0, 1.0 / 1000, 0.000001
     end
   end
 
   def test_assert_in_epsilon
-    @assertion_count = 8
+    @assertion_count = 10
 
     @tc.assert_in_epsilon 10000, 9991
     @tc.assert_in_epsilon 9991, 10000
@@ -647,14 +764,24 @@ class TestMiniTestUnitTestCase < MiniTest::Unit::TestCase
     @tc.assert_in_epsilon 9999.1, 10000, 0.0001
     @tc.assert_in_epsilon 1.0, 1.0001, 0.0001
     @tc.assert_in_epsilon 1.0001, 1.0, 0.0001
+
+    @tc.assert_in_epsilon(-1, -1)
+    @tc.assert_in_epsilon(-10000, -9991)
   end
 
   def test_assert_in_epsilon_triggered
-    util_assert_triggered 'Expected 10000 - 9990 (10) to be < 9.99.' do
+    util_assert_triggered 'Expected |10000 - 9990| (10) to be < 9.99.' do
       @tc.assert_in_epsilon 10000, 9990
     end
   end
 
+  def test_assert_in_epsilon_triggered_negative_case
+    x = RUBY18 ? "0.1" : "0.10000000000000009"
+    util_assert_triggered "Expected |-1.1 - -1| (#{x}) to be < 0.1." do
+      @tc.assert_in_epsilon(-1.1, -1, 0.1)
+    end
+  end
+
   def test_assert_includes
     @assertion_count = 2
 
@@ -697,7 +824,7 @@ class TestMiniTestUnitTestCase < MiniTest::Unit::TestCase
     @tc.assert_match(/\w+/, "blah blah blah")
   end
 
-  def test_assert_match_object
+  def test_assert_match_matcher_object
     @assertion_count = 2
 
     pattern = Object.new
@@ -706,6 +833,15 @@ class TestMiniTestUnitTestCase < MiniTest::Unit::TestCase
     @tc.assert_match pattern, 5
   end
 
+  def test_assert_match_matchee_to_str
+    @assertion_count = 2
+
+    obj = Object.new
+    def obj.to_str; "blah" end
+
+    @tc.assert_match "blah", obj
+  end
+
   def test_assert_match_object_triggered
     @assertion_count = 2
 
@@ -739,6 +875,13 @@ class TestMiniTestUnitTestCase < MiniTest::Unit::TestCase
     @tc.assert_operator 2, :>, 1
   end
 
+  def test_assert_operator_bad_object
+    bad = Object.new
+    def bad.==(other) true end
+
+    @tc.assert_operator bad, :equal?, bad
+  end
+
   def test_assert_operator_triggered
     util_assert_triggered "Expected 2 to be < 1." do
       @tc.assert_operator 2, :<, 1
@@ -754,6 +897,15 @@ class TestMiniTestUnitTestCase < MiniTest::Unit::TestCase
     end
   end
 
+  def test_assert_output_both_regexps
+    @assertion_count = 4
+
+    @tc.assert_output(/y.y/, /bl.h/) do
+      print "yay"
+      $stderr.print "blah"
+    end
+  end
+
   def test_assert_output_err
     @tc.assert_output nil, "blah" do
       $stderr.print "blah"
@@ -775,7 +927,7 @@ class TestMiniTestUnitTestCase < MiniTest::Unit::TestCase
   end
 
   def test_assert_output_triggered_both
-    util_assert_triggered util_msg("yay", "boo", "In stdout") do
+    util_assert_triggered util_msg("blah", "blah blah", "In stderr") do
       @tc.assert_output "yay", "blah" do
         print "boo"
         $stderr.print "blah blah"
@@ -799,12 +951,28 @@ class TestMiniTestUnitTestCase < MiniTest::Unit::TestCase
     end
   end
 
+  def test_assert_predicate
+    @tc.assert_predicate "", :empty?
+  end
+
+  def test_assert_predicate_triggered
+    util_assert_triggered 'Expected "blah" to be empty?.' do
+      @tc.assert_predicate "blah", :empty?
+    end
+  end
+
   def test_assert_raises
     @tc.assert_raises RuntimeError do
       raise "blah"
     end
   end
 
+  def test_assert_raises_module
+    @tc.assert_raises MyModule do
+      raise AnError
+    end
+  end
+
   ##
   # *sigh* This is quite an odd scenario, but it is from real (albeit
   # ugly) test code in ruby-core:
@@ -825,12 +993,6 @@ class TestMiniTestUnitTestCase < MiniTest::Unit::TestCase
     end
   end
 
-  def test_assert_raises_module
-    @tc.assert_raises MyModule do
-      raise AnError
-    end
-  end
-
   def test_assert_raises_triggered_different
     e = assert_raises MiniTest::Assertion do
       @tc.assert_raises RuntimeError do
@@ -838,15 +1000,17 @@ class TestMiniTestUnitTestCase < MiniTest::Unit::TestCase
       end
     end
 
-    expected = "[RuntimeError] exception expected, not
-Class: <SyntaxError>
-Message: <\"icky\">
----Backtrace---
-FILE:LINE:in `test_assert_raises_triggered_different'
----------------"
+    expected = clean <<-EOM.chomp
+      [RuntimeError] exception expected, not
+      Class: <SyntaxError>
+      Message: <\"icky\">
+      ---Backtrace---
+      FILE:LINE:in `test_assert_raises_triggered_different'
+      ---------------
+    EOM
 
     actual = e.message.gsub(/^.+:\d+/, 'FILE:LINE')
-    actual.gsub!(/block \(\d+ levels\) in /, '') if RUBY_VERSION =~ /^1\.9/
+    actual.gsub!(/block \(\d+ levels\) in /, '') if RUBY_VERSION >= '1.9.0'
 
     assert_equal expected, actual
   end
@@ -858,18 +1022,20 @@ FILE:LINE:in `test_assert_raises_triggered_different'
       end
     end
 
-    expected = "XXX
-[RuntimeError] exception expected, not
-Class: <SyntaxError>
-Message: <\"icky\">
----Backtrace---
-FILE:LINE:in `test_assert_raises_triggered_different_msg'
----------------"
+    expected = clean <<-EOM
+      XXX.
+      [RuntimeError] exception expected, not
+      Class: <SyntaxError>
+      Message: <\"icky\">
+      ---Backtrace---
+      FILE:LINE:in `test_assert_raises_triggered_different_msg'
+      ---------------
+    EOM
 
     actual = e.message.gsub(/^.+:\d+/, 'FILE:LINE')
-    actual.gsub!(/block \(\d+ levels\) in /, '') if RUBY_VERSION =~ /^1\.9/
+    actual.gsub!(/block \(\d+ levels\) in /, '') if RUBY_VERSION >= '1.9.0'
 
-    assert_equal expected, actual
+    assert_equal expected.chomp, actual
   end
 
   def test_assert_raises_triggered_none
@@ -891,7 +1057,7 @@ FILE:LINE:in `test_assert_raises_triggered_different_msg'
       end
     end
 
-    expected = "XXX\nMiniTest::Assertion expected but nothing was raised."
+    expected = "XXX.\nMiniTest::Assertion expected but nothing was raised."
 
     assert_equal expected, e.message
   end
@@ -903,15 +1069,17 @@ FILE:LINE:in `test_assert_raises_triggered_different_msg'
       end
     end
 
-    expected = "[StandardError] exception expected, not
-Class: <AnError>
-Message: <\"AnError\">
----Backtrace---
-FILE:LINE:in `test_assert_raises_triggered_subclass'
----------------"
+    expected = clean <<-EOM.chomp
+      [StandardError] exception expected, not
+      Class: <AnError>
+      Message: <\"AnError\">
+      ---Backtrace---
+      FILE:LINE:in `test_assert_raises_triggered_subclass'
+      ---------------
+    EOM
 
     actual = e.message.gsub(/^.+:\d+/, 'FILE:LINE')
-    actual.gsub!(/block \(\d+ levels\) in /, '') if RUBY_VERSION =~ /^1\.9/
+    actual.gsub!(/block \(\d+ levels\) in /, '') if RUBY_VERSION >= '1.9.0'
 
     assert_equal expected, actual
   end
@@ -969,8 +1137,6 @@ FILE:LINE:in `test_assert_raises_triggered_subclass'
   end
 
   def test_assert_silent_triggered_err
-    @assertion_count = 2
-
     util_assert_triggered util_msg("", "blah blah", "In stderr") do
       @tc.assert_silent do
         $stderr.print "blah blah"
@@ -979,6 +1145,8 @@ FILE:LINE:in `test_assert_raises_triggered_subclass'
   end
 
   def test_assert_silent_triggered_out
+    @assertion_count = 2
+
     util_assert_triggered util_msg("", "blah blah", "In stdout") do
       @tc.assert_silent do
         print "blah blah"
@@ -987,14 +1155,14 @@ FILE:LINE:in `test_assert_raises_triggered_subclass'
   end
 
   def test_assert_throws
-    @tc.assert_throws(:blah) do
+    @tc.assert_throws :blah do
       throw :blah
     end
   end
 
   def test_assert_throws_different
     util_assert_triggered 'Expected :blah to have been thrown, not :not_blah.' do
-      @tc.assert_throws(:blah) do
+      @tc.assert_throws :blah do
         throw :not_blah
       end
     end
@@ -1002,7 +1170,7 @@ FILE:LINE:in `test_assert_raises_triggered_subclass'
 
   def test_assert_throws_unthrown
     util_assert_triggered 'Expected :blah to have been thrown.' do
-      @tc.assert_throws(:blah) do
+      @tc.assert_throws :blah do
         # do nothing
       end
     end
@@ -1030,10 +1198,15 @@ FILE:LINE:in `test_assert_raises_triggered_subclass'
     methods = MiniTest::Assertions.public_instance_methods
     methods.map! { |m| m.to_s } if Symbol === methods.first
 
-    ignores = %w(assert_block assert_no_match assert_not_equal
-                 assert_not_nil assert_not_same assert_nothing_raised
-                 assert_nothing_thrown assert_output assert_raise
-                 assert_raises assert_send assert_silent assert_throws)
+    # These don't have corresponding refutes _on purpose_. They're
+    # useless and will never be added, so don't bother.
+    ignores = %w[assert_block assert_output assert_raises assert_send
+                 assert_silent assert_throws]
+
+    # These are test/unit methods. I'm not actually sure why they're still here
+    ignores += %w[assert_no_match assert_not_equal assert_not_nil
+                  assert_not_same assert_nothing_raised
+                  assert_nothing_thrown assert_raise]
 
     asserts = methods.grep(/^assert/).sort - ignores
     refutes = methods.grep(/^refute/).sort - ignores
@@ -1042,21 +1215,31 @@ FILE:LINE:in `test_assert_raises_triggered_subclass'
     assert_empty asserts.map { |n| n.sub(/^assert/, 'refute') } - refutes
   end
 
-  def test_class_inherited
-    @assertion_count = 0
-
-    Object.const_set(:ATestCase, Class.new(MiniTest::Unit::TestCase))
-
-    assert_equal [ATestCase], MiniTest::Unit::TestCase.test_suites
-  end
-
   def test_class_test_suites
     @assertion_count = 0
 
-    Object.const_set(:ATestCase, Class.new(MiniTest::Unit::TestCase))
+    tc = Class.new(MiniTest::Unit::TestCase)
 
     assert_equal 1, MiniTest::Unit::TestCase.test_suites.size
-    assert_equal [ATestCase], MiniTest::Unit::TestCase.test_suites
+    assert_equal [tc], MiniTest::Unit::TestCase.test_suites
+  end
+
+  def test_expectation
+    @assertion_count = 2
+
+    @tc.assert_equal true, 1.must_equal(1)
+  end
+
+  def test_expectation_triggered
+    util_assert_triggered "Expected: 2\n  Actual: 1" do
+      1.must_equal 2
+    end
+  end
+
+  def test_expectation_with_a_message
+    util_assert_triggered "Expected: 2\n  Actual: 1" do
+      1.must_equal 2, ''
+    end
   end
 
   def test_flunk
@@ -1084,6 +1267,11 @@ FILE:LINE:in `test_assert_raises_triggered_subclass'
     @tc.pass
   end
 
+  def test_prints
+    printer = Class.new { extend MiniTest::Assertions }
+    @tc.assert_equal '"test"', printer.mu_pp(ImmutableString.new 'test')
+  end
+
   def test_refute
     @assertion_count = 2
 
@@ -1119,7 +1307,7 @@ FILE:LINE:in `test_assert_raises_triggered_subclass'
   end
 
   def test_refute_in_delta_triggered
-    util_assert_triggered 'Expected 0.0 - 0.001 (0.001) to not be < 0.1.' do
+    util_assert_triggered 'Expected |0.0 - 0.001| (0.001) to not be < 0.1.' do
       @tc.refute_in_delta 0.0, 1.0 / 1000, 0.1
     end
   end
@@ -1129,7 +1317,7 @@ FILE:LINE:in `test_assert_raises_triggered_subclass'
   end
 
   def test_refute_in_epsilon_triggered
-    util_assert_triggered 'Expected 10000 - 9991 (9) to not be < 10.0.' do
+    util_assert_triggered 'Expected |10000 - 9991| (9) to not be < 10.0.' do
       @tc.refute_in_epsilon 10000, 9991
       fail
     end
@@ -1177,7 +1365,7 @@ FILE:LINE:in `test_assert_raises_triggered_subclass'
     @tc.refute_match(/\d+/, "blah blah blah")
   end
 
-  def test_refute_match_object
+  def test_refute_match_matcher_object
     @assertion_count = 2
     @tc.refute_match Object.new, 5 # default #=~ returns false
   end
@@ -1211,10 +1399,27 @@ FILE:LINE:in `test_assert_raises_triggered_subclass'
     end
   end
 
+  def test_refute_predicate
+    @tc.refute_predicate "42", :empty?
+  end
+
+  def test_refute_predicate_triggered
+    util_assert_triggered 'Expected "" to not be empty?.' do
+      @tc.refute_predicate "", :empty?
+    end
+  end
+
   def test_refute_operator
     @tc.refute_operator 2, :<, 1
   end
 
+  def test_refute_operator_bad_object
+    bad = Object.new
+    def bad.==(other) true end
+
+    @tc.refute_operator true, :equal?, bad
+  end
+
   def test_refute_operator_triggered
     util_assert_triggered "Expected 2 to not be > 1." do
       @tc.refute_operator 2, :>, 1
@@ -1252,7 +1457,7 @@ FILE:LINE:in `test_assert_raises_triggered_subclass'
   def test_test_methods_random
     @assertion_count = 0
 
-    sample_test_case = Class.new(MiniTest::Unit::TestCase) do
+    sample_test_case = Class.new MiniTest::Unit::TestCase do
       def test_test1; assert "does not matter" end
       def test_test2; assert "does not matter" end
       def test_test3; assert "does not matter" end
@@ -1266,7 +1471,7 @@ FILE:LINE:in `test_assert_raises_triggered_subclass'
   def test_test_methods_sorted
     @assertion_count = 0
 
-    sample_test_case = Class.new(MiniTest::Unit::TestCase) do
+    sample_test_case = Class.new MiniTest::Unit::TestCase do
       def self.test_order; :sorted end
       def test_test3; assert "does not matter" end
       def test_test2; assert "does not matter" end
@@ -1277,8 +1482,30 @@ FILE:LINE:in `test_assert_raises_triggered_subclass'
     assert_equal expected, sample_test_case.test_methods
   end
 
+  def test_i_suck_and_my_tests_are_order_dependent_bang_sets_test_order_alpha
+    @assertion_count = 0
+
+    shitty_test_case = Class.new MiniTest::Unit::TestCase
+
+    shitty_test_case.i_suck_and_my_tests_are_order_dependent!
+
+    assert_equal :alpha, shitty_test_case.test_order
+  end
+
+  def test_i_suck_and_my_tests_are_order_dependent_bang_does_not_warn
+    @assertion_count = 0
+
+    shitty_test_case = Class.new MiniTest::Unit::TestCase
+
+    def shitty_test_case.test_order ; :lol end
+
+    assert_silent do
+      shitty_test_case.i_suck_and_my_tests_are_order_dependent!
+    end
+  end
+
   def util_assert_triggered expected, klass = MiniTest::Assertion
-    e = assert_raises(klass) do
+    e = assert_raises klass do
       yield
     end
 
@@ -1303,3 +1530,25 @@ FILE:LINE:in `test_assert_raises_triggered_subclass'
     MiniTest::Assertions.diff = old_diff
   end
 end
+
+class TestMiniTestGuard < MiniTest::Unit::TestCase
+  def test_mri_eh
+    assert self.class.mri? "ruby blah"
+    assert self.mri? "ruby blah"
+  end
+
+  def test_jruby_eh
+    assert self.class.jruby? "java"
+    assert self.jruby? "java"
+  end
+
+  def test_rubinius_eh
+    assert self.class.rubinius? "rbx"
+    assert self.rubinius? "rbx"
+  end
+
+  def test_windows_eh
+    assert self.class.windows? "mswin"
+    assert self.windows? "mswin"
+  end
+end
