@@ -7,9 +7,16 @@ class Complex_Test < Test::Unit::TestCase
   def setup
     @rational = defined?(Rational)
     if @rational
-      @keiju = Rational.instance_variable_get('@RCS_ID')
+      @keiju = Rational.instance_variables.include?(:@RCS_ID)
     end
-    @unify = defined?(Complex::Unify)
+    seps = [File::SEPARATOR, File::ALT_SEPARATOR].compact.map{|x| Regexp.escape(x)}.join("|")
+    @unify = $".grep(/(?:^|#{seps})mathn(?:\.(?:rb|so))?/).size != 0
+  end
+
+  def test_rationalize
+    assert_equal(1.quo(3), Complex(1/3.0, 0).rationalize, '[ruby-core:38885]')
+    assert_equal(1.quo(5), Complex(0.2, 0).rationalize, '[ruby-core:38885]')
+    assert_equal(5.quo(2), Complex(2.5, 0).rationalize(0), '[ruby-core:40667]')
   end
 
   def test_compsub
@@ -27,7 +34,7 @@ class Complex_Test < Test::Unit::TestCase
       c2 = c - 1
       assert_instance_of(ComplexSub, c2)
 
-      c3 = c - c
+      c3 = c - c2
       assert_instance_of(ComplexSub, c3)
 
       s = Marshal.dump(c)
@@ -36,6 +43,9 @@ class Complex_Test < Test::Unit::TestCase
       assert_instance_of(ComplexSub, c5)
     end
 
+    c1 = Complex(1)
+    assert_equal(c1.hash, c.hash, '[ruby-dev:38850]')
+    assert_equal([true, true], [c.eql?(c1), c1.eql?(c)])
   end
 
   def test_eql_p
@@ -71,6 +81,12 @@ class Complex_Test < Test::Unit::TestCase
 
     h[Complex(0.0,0.0)] = 9.0
     assert_equal(5, h.size)
+
+    if (0.0/0).nan? && !((0.0/0).eql?(0.0/0))
+      h = {}
+      3.times{h[Complex(0.0/0)] = 1}
+      assert_equal(3, h.size)
+    end
   end
 
   def test_freeze
@@ -120,10 +136,17 @@ class Complex_Test < Test::Unit::TestCase
     if @rational && !@keiju
       assert_equal(Complex(1,1),Complex('3/3','3/3'))
     end
-    assert_raise(ArgumentError){Complex(nil)}
-    assert_raise(ArgumentError){Complex(Object.new)}
+    assert_raise(TypeError){Complex(nil)}
+    assert_raise(TypeError){Complex(Object.new)}
     assert_raise(ArgumentError){Complex()}
     assert_raise(ArgumentError){Complex(1,2,3)}
+
+    if (0.0/0).nan?
+      assert_nothing_raised{Complex(0.0/0)}
+    end
+    if (1.0/0).infinite?
+      assert_nothing_raised{Complex(1.0/0)}
+    end
   end
 
   def test_attr
@@ -475,7 +498,7 @@ class Complex_Test < Test::Unit::TestCase
     assert_raise(NoMethodError){Complex(1,1) <=> Complex(1,1)}
   end
 
-  def test_equal
+  def test_eqeq
     assert(Complex(1,0) == Complex(1))
     assert(Complex(-1,0) == Complex(-1))
 
@@ -483,6 +506,38 @@ class Complex_Test < Test::Unit::TestCase
     assert_equal(true, Complex(2,1) != Complex(1))
     assert_equal(false, Complex(1) == nil)
     assert_equal(false, Complex(1) == '')
+
+    nan = 0.0 / 0
+    if nan.nan? && nan != nan
+      assert_equal(false, Complex(nan, 0) == Complex(nan, 0))
+      assert_equal(false, Complex(0, nan) == Complex(0, nan))
+      assert_equal(false, Complex(nan, nan) == Complex(nan, nan))
+    end
+  end
+
+  def test_coerce
+    assert_equal([Complex(2),Complex(1)], Complex(1).coerce(2))
+    assert_equal([Complex(2.2),Complex(1)], Complex(1).coerce(2.2))
+    assert_equal([Complex(Rational(2)),Complex(1)],
+		 Complex(1).coerce(Rational(2)))
+    assert_equal([Complex(2),Complex(1)], Complex(1).coerce(Complex(2)))
+  end
+
+  class ObjectX
+    def + (x) Rational(1) end
+    alias - +
+    alias * +
+    alias / +
+    alias quo +
+    alias ** +
+    def coerce(x) [x, Complex(1)] end
+  end
+
+  def test_coerce2
+    x = ObjectX.new
+    %w(+ - * / quo **).each do |op|
+      assert_kind_of(Numeric, Complex(1).__send__(op, x))
+    end
   end
 
   def test_unify
@@ -491,7 +546,7 @@ class Complex_Test < Test::Unit::TestCase
       assert_instance_of(Fixnum, Complex(1,2) - Complex(1,2))
       assert_instance_of(Fixnum, Complex(1,2) * 0)
       assert_instance_of(Fixnum, Complex(1,2) / Complex(1,2))
-      assert_instance_of(Fixnum, Complex(1,2).div(Complex(1,2)))
+#      assert_instance_of(Fixnum, Complex(1,2).div(Complex(1,2)))
       assert_instance_of(Fixnum, Complex(1,2).quo(Complex(1,2)))
 #      assert_instance_of(Fixnum, Complex(1,2) ** 0) # mathn's bug
     end
@@ -564,6 +619,16 @@ class Complex_Test < Test::Unit::TestCase
       assert_equal('1-2/3i', Complex(1,Rational(-2,3)).to_s)
       assert_equal('-1-2/3i', Complex(-1,Rational(-2,3)).to_s)
     end
+
+    nan = 0.0 / 0
+    inf = 1.0 / 0
+    if nan.nan?
+      assert_equal('NaN+NaN*i', Complex(nan,nan).to_s)
+    end
+    if inf.infinite?
+      assert_equal('Infinity+Infinity*i', Complex(inf,inf).to_s)
+      assert_equal('Infinity-Infinity*i', Complex(inf,-inf).to_s)
+    end
   end
 
   def test_inspect
@@ -591,6 +656,22 @@ class Complex_Test < Test::Unit::TestCase
       assert_equal(c, c2)
       assert_instance_of(Complex, c2)
     end
+
+    bug3656 = '[ruby-core:31622]'
+    c = Complex(1,2)
+    c.freeze
+    assert(c.frozen?)
+    result = c.marshal_load([2,3]) rescue :fail
+    assert_equal(:fail, result, bug3656)
+    assert_equal(Complex(1,2), c)
+  end
+
+  def test_marshal_compatibility
+    bug6625 = '[ruby-core:45775]'
+    dump = "\x04\x08o:\x0cComplex\x07:\x0a@reali\x06:\x0b@imagei\x07"
+    assert_nothing_raised(bug6625) do
+      assert_equal(Complex(1, 2), Marshal.load(dump), bug6625)
+    end
   end
 
   def test_parse
@@ -625,6 +706,15 @@ class Complex_Test < Test::Unit::TestCase
     assert_equal(Complex(0.0,3.0), '3.0i'.to_c)
     assert_equal(Complex(0.0,-3.0), '-3.0i'.to_c)
 
+    assert_equal(Complex(5.1), '5.1'.to_c)
+    assert_equal(Complex(-5.2), '-5.2'.to_c)
+    assert_equal(Complex(5.3,3.4), '5.3+3.4i'.to_c)
+    assert_equal(Complex(-5.5,3.6), '-5.5+3.6i'.to_c)
+    assert_equal(Complex(5.3,-3.4), '5.3-3.4i'.to_c)
+    assert_equal(Complex(-5.5,-3.6), '-5.5-3.6i'.to_c)
+    assert_equal(Complex(0.0,3.1), '3.1i'.to_c)
+    assert_equal(Complex(0.0,-3.2), '-3.2i'.to_c)
+
     assert_equal(Complex(5.0), '5e0'.to_c)
     assert_equal(Complex(-5.0), '-5e0'.to_c)
     assert_equal(Complex(5.0,3.0), '5e0+3e0i'.to_c)
@@ -634,6 +724,15 @@ class Complex_Test < Test::Unit::TestCase
     assert_equal(Complex(0.0,3.0), '3e0i'.to_c)
     assert_equal(Complex(0.0,-3.0), '-3e0i'.to_c)
 
+    assert_equal(Complex(5e1), '5e1'.to_c)
+    assert_equal(Complex(-5e2), '-5e2'.to_c)
+    assert_equal(Complex(5e3,3e4), '5e003+3e4i'.to_c)
+    assert_equal(Complex(-5e5,3e6), '-5e5+3e006i'.to_c)
+    assert_equal(Complex(5e3,-3e4), '5e003-3e4i'.to_c)
+    assert_equal(Complex(-5e5,-3e6), '-5e5-3e006i'.to_c)
+    assert_equal(Complex(0.0,3e1), '3e1i'.to_c)
+    assert_equal(Complex(0.0,-3e2), '-3e2i'.to_c)
+
     assert_equal(Complex(0.33), '.33'.to_c)
     assert_equal(Complex(0.33), '0.33'.to_c)
     assert_equal(Complex(-0.33), '-.33'.to_c)
@@ -676,6 +775,15 @@ class Complex_Test < Test::Unit::TestCase
     assert_equal(Complex(0.0,3.0), Complex('3.0i'))
     assert_equal(Complex(0.0,-3.0), Complex('-3.0i'))
 
+    assert_equal(Complex(5.1), Complex('5.1'))
+    assert_equal(Complex(-5.2), Complex('-5.2'))
+    assert_equal(Complex(5.3,3.4), Complex('5.3+3.4i'))
+    assert_equal(Complex(-5.5,3.6), Complex('-5.5+3.6i'))
+    assert_equal(Complex(5.3,-3.4), Complex('5.3-3.4i'))
+    assert_equal(Complex(-5.5,-3.6), Complex('-5.5-3.6i'))
+    assert_equal(Complex(0.0,3.1), Complex('3.1i'))
+    assert_equal(Complex(0.0,-3.2), Complex('-3.2i'))
+
     assert_equal(Complex(5.0), Complex('5e0'))
     assert_equal(Complex(-5.0), Complex('-5e0'))
     assert_equal(Complex(5.0,3.0), Complex('5e0+3e0i'))
@@ -685,6 +793,15 @@ class Complex_Test < Test::Unit::TestCase
     assert_equal(Complex(0.0,3.0), Complex('3e0i'))
     assert_equal(Complex(0.0,-3.0), Complex('-3e0i'))
 
+    assert_equal(Complex(5e1), Complex('5e1'))
+    assert_equal(Complex(-5e2), Complex('-5e2'))
+    assert_equal(Complex(5e3,3e4), Complex('5e003+3e4i'))
+    assert_equal(Complex(-5e5,3e6), Complex('-5e5+3e006i'))
+    assert_equal(Complex(5e3,-3e4), Complex('5e003-3e4i'))
+    assert_equal(Complex(-5e5,-3e6), Complex('-5e5-3e006i'))
+    assert_equal(Complex(0.0,3e1), Complex('3e1i'))
+    assert_equal(Complex(0.0,-3e2), Complex('-3e2i'))
+
     assert_equal(Complex(0.33), Complex('.33'))
     assert_equal(Complex(0.33), Complex('0.33'))
     assert_equal(Complex(-0.33), Complex('-.33'))
@@ -741,17 +858,19 @@ class Complex_Test < Test::Unit::TestCase
 
   def test_respond
     c = Complex(1,1)
+    assert_equal(false, c.respond_to?(:%))
     assert_equal(false, c.respond_to?(:<))
     assert_equal(false, c.respond_to?(:<=))
     assert_equal(false, c.respond_to?(:<=>))
     assert_equal(false, c.respond_to?(:>))
     assert_equal(false, c.respond_to?(:>=))
     assert_equal(false, c.respond_to?(:between?))
-#    assert_equal(false, c.respond_to?(:div)) # ?
+    assert_equal(false, c.respond_to?(:div))
     assert_equal(false, c.respond_to?(:divmod))
     assert_equal(false, c.respond_to?(:floor))
     assert_equal(false, c.respond_to?(:ceil))
     assert_equal(false, c.respond_to?(:modulo))
+    assert_equal(false, c.respond_to?(:remainder))
     assert_equal(false, c.respond_to?(:round))
     assert_equal(false, c.respond_to?(:step))
     assert_equal(false, c.respond_to?(:tunrcate))
@@ -794,13 +913,13 @@ class Complex_Test < Test::Unit::TestCase
 
   def test_to_c
     c = nil.to_c
-    assert_equal([0,0] , [c.real, c.imag])
+    assert_equal([0,0], [c.real, c.imag])
 
     c = 0.to_c
-    assert_equal([0,0] , [c.real, c.imag])
+    assert_equal([0,0], [c.real, c.imag])
 
     c = 1.to_c
-    assert_equal([1,0] , [c.real, c.imag])
+    assert_equal([1,0], [c.real, c.imag])
 
     c = 1.1.to_c
     assert_equal([1.1, 0], [c.real, c.imag])
@@ -812,6 +931,13 @@ class Complex_Test < Test::Unit::TestCase
 
     c = Complex(1,2).to_c
     assert_equal([1, 2], [c.real, c.imag])
+
+    if (0.0/0).nan?
+      assert_nothing_raised{(0.0/0).to_c}
+    end
+    if (1.0/0).infinite?
+      assert_nothing_raised{(1.0/0).to_c}
+    end
   end
 
   def test_supp
@@ -844,6 +970,13 @@ class Complex_Test < Test::Unit::TestCase
     assert_equal(0, 1.0.angle)
     assert_equal(0, 1.0.phase)
 
+    if (0.0/0).nan?
+      nan = 0.0/0
+      assert(nan.arg.equal?(nan))
+      assert(nan.angle.equal?(nan))
+      assert(nan.phase.equal?(nan))
+    end
+
     assert_equal(Math::PI, -1.arg)
     assert_equal(Math::PI, -1.angle)
     assert_equal(Math::PI, -1.phase)
@@ -898,7 +1031,7 @@ class Complex_Test < Test::Unit::TestCase
     end
     assert_equal(Complex(0.5,1.0), Complex(1,2).quo(2))
 
-    unless $".grep(/(\A|\/)complex/).empty?
+    unless $".grep(/(?:\A|(?<!add)\/)complex/).empty?
       assert_equal(Complex(0,2), Math.sqrt(-4.0))
 #      assert_equal(true, Math.sqrt(-4.0).inexact?)
       assert_equal(Complex(0,2), Math.sqrt(-4))
@@ -1019,6 +1152,8 @@ class Complex_Test < Test::Unit::TestCase
       assert_equal(Complex(1), 1 ** Complex(1))
     end
     assert_equal('-1.0-0.0i', Complex(-1.0, -0.0).to_s)
+    assert_in_delta(Math::PI, Complex(-0.0).arg, 0.001)
+    assert_equal(Complex(2e3, 2e4), '2e3+2e4i'.to_c)
   end
 
   def test_known_bug
